@@ -39,6 +39,11 @@
 #' \itemize{
 #'  \item `"system"` -
 #'    Query \code{\link[parallel]{detectCores}(logical = logical)}.
+#'  \item `"cgroups.cpuset"` -
+#'    On Unix, query control group (cgroup) value \code{cpuset.set}.
+#'  \item `"cgroups.cpuquota"` -
+#'    On Unix, query control group (cgroup) value
+#'    \code{cpu.cfs_quota_us} / \code{cpu.cfs_period_us}.
 #'  \item `"nproc"` -
 #'    On Unix, query system command \code{nproc}.
 #'  \item `"mc.cores"` -
@@ -54,6 +59,12 @@
 #'    package is loaded).  The \option{mc.cores} option is used by for
 #'    instance \code{\link[=mclapply]{mclapply}()} of the \pkg{parallel}
 #'    package.
+#'  \item `"BiocParallel"` -
+#'    Query environment variables \env{BIOCPARALLEL_WORKER_NUMBER} (integer),
+#'    which is defined and used by **BiocParallel** (>= 1.27.2), and
+#'    \env{BBS_HOME} (logical) used by the Bioconductor Build System. If the
+#'    former is set, this is the number of cores considered.  If the latter
+#'    is set, then a maximum of 4 cores is considered.
 #'  \item `"PBS"` -
 #'    Query TORQUE/PBS environment variables \env{PBS_NUM_PPN} and \env{NCPUS}.
 #'    Depending on PBS system configuration, these _resource_
@@ -154,7 +165,7 @@
 #'
 #' @importFrom parallel detectCores
 #' @export
-availableCores <- function(constraints = NULL, methods = getOption2("parallelly.availableCores.methods", c("system", "nproc", "mc.cores", "_R_CHECK_LIMIT_CORES_", "PBS", "SGE", "Slurm", "LSF", "fallback", "custom")), na.rm = TRUE, logical = getOption2("parallelly.availableCores.logical", TRUE), default = c(current = 1L), which = c("min", "max", "all"), omit = getOption2("parallelly.availableCores.omit", 0L)) {
+availableCores <- function(constraints = NULL, methods = getOption2("parallelly.availableCores.methods", c("system", "cgroups.cpuset", "cgroups.cpuquota", "nproc", "mc.cores", "BiocParallel", "_R_CHECK_LIMIT_CORES_", "PBS", "SGE", "Slurm", "LSF", "fallback", "custom")), na.rm = TRUE, logical = getOption2("parallelly.availableCores.logical", TRUE), default = c(current = 1L), which = c("min", "max", "all"), omit = getOption2("parallelly.availableCores.omit", 0L)) {
   ## Local functions
   getenv <- function(name, mode = "integer") {
     value <- trim(getEnvVar2(name, default = NA_character_))
@@ -253,6 +264,9 @@ availableCores <- function(constraints = NULL, methods = getOption2("parallelly.
     } else if (method == "mc.cores+1") {
       ## Number of cores by option defined by 'parallel' package
       n <- getopt("mc.cores") + 1L
+    } else if (method == "BiocParallel") {
+      n <- getenv("BIOCPARALLEL_WORKER_NUMBER")
+      if (nzchar(Sys.getenv("BBS_HOME"))) n <- min(n, 4L, na.rm = TRUE)
     } else if (method == "_R_CHECK_LIMIT_CORES_") {
       ## A flag set by R CMD check for constraining number of
       ## cores allowed to be use in package tests.  Here we
@@ -266,8 +280,19 @@ availableCores <- function(constraints = NULL, methods = getOption2("parallelly.
     } else if (method == "system") {
       ## Number of cores available according to parallel::detectCores()
       n <- detectCores(logical = logical)
+    } else if (method == "cgroups.cpuset") {
+      ## Number of cores according to Unix Cgroups CPU set
+      n <- length(getCGroupsCpuSet())
+      if (n == 0L) n <- NA_integer_
+    } else if (method == "cgroups.cpuquota") {
+      ## Number of cores according to Unix Cgroups CPU quota
+      n <- getCGroupsCpuQuota()
+      if (!is.na(n)) {
+        n <- as.integer(floor(n + 0.5))
+	if (n == 0L) n <- 1L  ## If CPU quota < 0.5, round up to one CPU
+      }
     } else if (method == "nproc") {
-      ## Number of cores according Unix 'nproc'
+      ## Number of cores according to Unix 'nproc'
       n <- getNproc()
     } else if (method == "fallback") {
       ## Number of cores available according to parallelly.availableCores.fallback
@@ -320,12 +345,20 @@ availableCores <- function(constraints = NULL, methods = getOption2("parallelly.
     ## options are explicitly set / available.
     idx_fallback <- which(names(ncores) == "fallback")
     if (length(idx_fallback) == 1) {
-      ## Use only if 'system' and 'nproc' are the only other options
-      ignore <- c("system", "nproc")
-      if (length(setdiff(names(ncores), c("fallback", ignore))) == 0) {
+      ## Use 'fallback' if and only there are only "special" options specified
+      special <- c("system", "cgroups.cpuset", "cgroups.cpuquote", "nproc")
+      others <- setdiff(names(ncores), c("fallback", special))
+      use_fallback <- (length(others) == 0L)
+
+      ## ... and all the "special" options agree. If one of them disagree,
+      ## it's likely that cgroups limits the CPUs
+      if (use_fallback && any(ncores[special] < ncores["system"], na.rm = TRUE)) {
+        use_fallback <- FALSE
+      }
+      
+      if (use_fallback) {
         ncores <- ncores[idx_fallback]
       } else {
-        ## ... otherwise, ignore 'fallback'.
         ncores <- ncores[-idx_fallback]
       }
     }
@@ -348,6 +381,20 @@ availableCores <- function(constraints = NULL, methods = getOption2("parallelly.
     }
   }
 
+  ## Override the minimum of one (1) core?
+  min <- getOption2("parallelly.availableCores.min", 1L)
+  if (length(min) != 1L || !is.numeric(min)) {
+    stop(sprintf("Option %s is not numeric: %s", sQuote("parallelly.availableCores.min"), mode(min)))
+  } else if (!is.finite(min) || min < 1L) {
+    stop(sprintf("Option %s must be an integer greater than one: %d", sQuote("parallelly.availableCores.min"), min))
+  } else if (min > detectCores(logical = logical)) {
+    stop(sprintf("Option %s must not be greater than the number cores on the system: %d > %d", sQuote("parallelly.availableCores.min"), min, detectCores(logical = logical)))
+  } else {
+    idxs <- which(ncores < min)
+    ncores[idxs] <- as.integer(floor(min))
+    names(ncores)[idxs] <- paste(names(ncores)[idxs], "*", sep = "")
+  }
+
   ## Omit some of the cores?
   if (omit > 0L) {
     ncores <- ncores - omit
@@ -382,11 +429,11 @@ getNproc <- function(ignore = c("OMP_NUM_THREADS", "OMP_THREAD_LIMIT")) {
   if (length(m) == 0L) return(NA_integer_)
 
   for (cmd in systems[[m]]) {
-    res <- tryCatch({
-      suppressWarnings(system(cmd, intern=TRUE))
-    }, error = function(e) NULL)
-    res <- gsub("(^[[:space:]]+|[[:space:]]+$)", "", res[1])
-    if (grepl("^[1-9]$", res)) return(as.integer(res))
+    tryCatch({
+      res <- suppressWarnings(system(cmd, intern=TRUE))
+      res <- gsub("(^[[:space:]]+|[[:space:]]+$)", "", res[1])
+      if (grepl("^[[:digit:]]+$", res)) return(as.integer(res))
+    }, error = identity)
   }
   
   NA_integer_
